@@ -24,10 +24,13 @@ import re
 import copy
 import shutil
 import black
+import logging
+
 from pathlib import Path
 from functools import partial
 from itertools import zip_longest
 from typing import List
+from black import InvalidInput
 
 try:
     from subprocess import getoutput
@@ -98,6 +101,8 @@ CODEBLOCK = "code-block"
 CODE = "code"
 PACKAGE = "package"
 
+# Disable blib2to3 output that clutters debugging log.
+logging.getLogger("blib2to3").setLevel(logging.ERROR)
 
 # Run sphinx-build with Markdown builder in the plugin.
 def run_sphinx_markdown():
@@ -671,6 +676,19 @@ def _extract_docstring_info(summary_info, summary, name):
     return top_summary
 
 
+# Returns appropriate product name to display for given full name of entry.
+def extract_product_name(name):
+    if 'google.cloud' in name:
+        product_name = '.'.join(name.split('.')[2:])
+    elif 'google' in name:
+        product_name = '.'.join(name.split('.')[1:])
+    else:
+        # Use the short name for other formats.
+        product_name = name.split('.')[-1]
+
+    return product_name
+
+
 def _create_datam(app, cls, module, name, _type, obj, lines=None):
     """
     Build the data structure for an autodoc class
@@ -861,7 +879,8 @@ def _create_datam(app, cls, module, name, _type, obj, lines=None):
 
     # If there is no summary, add a short snippet.
     else:
-        datam['summary'] = f"API documentation for `{short_name}` {_type}."
+        product_name = extract_product_name(name)
+        datam['summary'] = f"API documentation for `{product_name}` {_type}."
 
     if args or sig or summary_info:
         datam['syntax'] = {}
@@ -984,7 +1003,11 @@ def format_code(code):
 def process_signature(app, _type, name, obj, options, signature, return_annotation):
     if signature:
         short_name = name.split('.')[-1]
-        signature = format_code(short_name + signature)
+        signature = short_name + signature
+        try:
+            signature = format_code(signature)
+        except InvalidInput as e:
+            print(f"Could not format the given code: \n{e})")
         app.env.docfx_signature_funcs_methods[name] = signature
 
 
@@ -1211,7 +1234,7 @@ def parse_markdown_header(header_line, prev_line):
         if not header_line[header_line.index(h1_header_prefix)+2].isspace() and \
             len(header_line) > 2:
 
-            return header_line.strip("#").strip()
+            return header_line[header_line.index(h1_header_prefix):].strip("#").strip()
 
     elif "=" in header_line:
         # Check if we're inspecting an empty or undefined lines.
@@ -1245,6 +1268,52 @@ def extract_header_from_markdown(mdfile_iterator):
     return mdfile_name
 
 
+# For a given markdown file, adds syntax highlighting to code blocks.
+def highlight_md_codeblocks(mdfile):
+    fence = '```'
+    fence_with_python = '```python'
+    new_lines = []
+
+    with open(mdfile) as mdfile_iterator:
+        file_content = mdfile_iterator.read()
+        # If there is an odd number of code block annotations, do not syntax
+        # highlight.
+        if file_content.count(fence) % 2 != 0:
+            print(f'{mdfile_iterator.name} contains wrong format of code blocks. Skipping syntax highlighting.')
+            return
+        # Retrieve code block positions to replace
+        codeblocks = [[m.start(), m.end()] for m in re.finditer(
+                                                      fence,
+                                                      file_content)]
+
+        # This is equivalent to grabbing every odd index item.
+        codeblocks = codeblocks[::2]
+        # Used to store code blocks that come without language indicators.
+        blocks_without_indicators = []
+
+        # Check if the fence comes without a language indicator. If so, include
+        # this to a list to render.
+        for start, end in codeblocks:
+            if file_content[end] == '\n':
+                blocks_without_indicators.append([start, end])
+
+        # Stitch content that does not need to be parsed, and replace with
+        # `fence_with_python` for parsed portions.
+        prev_start = prev_end = 0
+        for start, end in blocks_without_indicators:
+            new_lines.append(file_content[prev_end:start])
+            new_lines.append(fence_with_python)
+            prev_start, prev_end = start, end
+
+        # Include rest of the content.
+        new_lines.append(file_content[prev_end:])
+
+    # Overwrite with newly parsed content.
+    with open(mdfile, 'w') as mdfile_iterator:
+        new_content = ''.join(new_lines)
+        mdfile_iterator.write(new_content)
+
+
 # Given generated markdown files, incorporate them into the docfx_yaml output.
 # The markdown file metadata will be added to top level of the TOC.
 def find_markdown_pages(app, outdir):
@@ -1258,9 +1327,6 @@ def find_markdown_pages(app, outdir):
 
         "readme.md",    # README does not seem to work in cloud site
                         # See https://github.com/googleapis/sphinx-docfx-yaml/issues/107.
-
-        "upgrading.md", # Currently the formatting breaks, will need to come back to it.
-                        # See https://github.com/googleapis/sphinx-docfx-yaml/issues/108.
     ]
 
     markdown_dir = Path(app.builder.outdir).parent / "markdown"
@@ -1271,6 +1337,7 @@ def find_markdown_pages(app, outdir):
     # For each file, if it is a markdown file move to the top level pages.
     for mdfile in markdown_dir.iterdir():
         if mdfile.is_file() and mdfile.name.lower() not in files_to_ignore:
+            highlight_md_codeblocks(markdown_dir / mdfile.name)
             shutil.copy(mdfile, f"{outdir}/{mdfile.name.lower()}")
 
             # Extract the header name for TOC.
@@ -1423,7 +1490,8 @@ def build_finished(app, exception):
         if 'source' in obj and 'path' in obj['source'] and obj['source']['path']:
             if obj['source']['path'].endswith(INITPY):
                 obj['type'] = 'subPackage'
-                obj['summary'] = "API documentation for `{}` package.".format(obj['name'])
+                product_name = extract_product_name(obj['fullName'])
+                obj['summary'] = f"API documentation for `{product_name}` package."
                 return
 
         for child_uid in obj['children']:
